# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from elastic.party.models import Party
import mptt
from mptt.models import MPTTModel, TreeForeignKey, TreeManager
from django.core.exceptions import ObjectDoesNotExist
from elastic.product.models import Product
from django.db import transaction
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.utils import timezone
from django.utils import translation
from django.conf import settings
import uuid
from elastic.product.exceptions import *
from django.template import Context, Template
from elastic.workflow.models import Workflow, Schema
from django.db.models.signals import pre_delete, post_save
from django.dispatch import receiver

__author__ = "Igor S. Kovalenko"
__contact__ = "kovalenko@sb-soft.biz"
__site__ = "http://www.elastic-trade-server.org"
__year__ = "2015"
__description__ = "Stock package"


@receiver(post_save, dispatch_uid='stock_post_save')
def post_save_handler(sender, instance, **kwargs):
    if sender in [Consignment, RevaluationAct]:
        # Удаляем и заново создаем рабочий процесс (workflow) связанный с документом
        instance.get_workflow().delete()
        instance.get_workflow().start()


@receiver(pre_delete, dispatch_uid='stock_pre_delete')
def pre_delete_handler(sender, instance, **kwargs):
    if sender in [Consignment, RevaluationAct]:
        # Удаляем рабочий процесс (workflow) связанный с документом
        instance.get_workflow().delete()


class StockLocationManager(TreeManager):

    @staticmethod
    def get_product_price_cost_on_location(location_code, sku):
        return StockLocation.objects.get(code=location_code).get_product_price_cost(sku)

    @staticmethod
    def get_product_rest_on_location(location_code, sku):
        return StockLocation.objects.get(code=location_code).get_product_rest(sku)


class StockLocation(MPTTModel):
    class Meta:
        verbose_name = _('Stock location')
        verbose_name_plural = _('Stock locations')

    class MPTTMeta:
        parent_attr = 'parent'

    TYPE = (
        ('WH', _('Warehouse')),
        ('CUS', _('Customer')),
        ('SUP', _('Supplier')),
        ('IN', _('Input')),
        ('OUT', _('Output')),
        ('STO', _('Store')),
    )

    parent = TreeForeignKey('self', verbose_name=_('Parent'), null=True, blank=True, related_name='children')
    code = models.CharField(verbose_name=_('Code'), max_length=128, db_index=True)
    name = models.CharField(max_length=255, verbose_name=_('Name'))
    type_location = models.CharField(max_length=3, choices=TYPE, default='WH', verbose_name=_('Type location'))
    party = models.ForeignKey(Party, null=True, blank=True, verbose_name=_('Party'))

    objects = StockLocationManager()

    def __unicode__(self):
        if self.type_location == 'WH':
            return self.name
        root = self.get_root()
        return u"%s/%s" % (root.name, self.name)

    def get_product_price_cost(self, sku):
        product = Product.objects.get(sku=sku)
        try:
            return RestRegister.objects.get(location=self, product=product).actual_price_cost
        except ObjectDoesNotExist:
            return 0

    def get_product_rest(self, sku):
        product = Product.objects.get(sku=sku)
        try:
            register = RestRegister.objects.get(location=self, product=product)
            # if product.serial_mark in ['accounted_party', 'product_party', ]:
            #     pass
            # if product.serial_mark == 'none':
            #     pass
            return register.actual_quantity
        except ObjectDoesNotExist:
            return 0

    def save(self, *args, **kwargs):
        if not self.id and not self.parent:
            self.type_location = "WH"  # Корень всегда считается складом
        super(StockLocation, self).save(*args, **kwargs)


mptt.register(StockLocation)


class Consignment(models.Model):
    """
    ТОРГ-13 "Накладная на внутреннее перемещение, передачу товаров, тары"
    """
    class Meta:
        verbose_name = _('Consignment')
        verbose_name_plural = _('Consignments')

    #: Номер документа
    doc_no = models.CharField(verbose_name=_('Doc no'), max_length=128, db_index=True)

    #: Внутренний код документа
    code = models.CharField(max_length=128, verbose_name=_('Code'))

    #: Дата/время создания
    creation_date = models.DateTimeField(verbose_name=_('Creation date'))

    #: Дата запланированного прихода
    planned_date = models.DateTimeField(null=True, blank=True, verbose_name=_('Planned date'))

    #: Дата исполнения
    effective_date = models.DateTimeField(null=True, blank=True, verbose_name=_('Effective date'))

    #: Дата/время последней модификации
    last_modify_date = models.DateTimeField(verbose_name=_('Last modify date'))

    #: Метка узла-создателя документа
    created_by = models.CharField(max_length=64, verbose_name=_('Node creator'))

    #: Откуда (наименование)
    from_location_name = models.CharField(max_length=255, verbose_name=_('From location name'))

    #: Откуда (код)
    from_location_code = models.CharField(max_length=128, verbose_name=_('From location code'))

    #: Куда (наименованеи)
    to_location_name = models.CharField(max_length=255, verbose_name=_('To location name'))

    #: Куда (код)
    to_location_code = models.CharField(max_length=128, verbose_name=_('To location code'))

    #: Примечание
    description = models.TextField(null=True, blank=True, verbose_name=_('Description'))

    def __unicode__(self):
        tmpl = Template("{% load tz %} {{ effective_date|localtime}}")
        creation_date = tmpl.render(Context({'effective_date': self.creation_date}))
        return u"#{0} {1} {2}".format(self.doc_no, _('at'), creation_date)

    def get_workflow(self):
        # Регистрируем/загружаем связанный поток исполнения
        workflow_schema = Schema.objects.get_or_create(
                code='goods_reception',
                defaults={
                    "name": 'Оприходование товара',
                    "code": 'goods_reception',
                })

        workflow_instance = Workflow.objects.get_or_create(
                doc_id=self.id,
                defaults={
                    "doc_object": self,
                    "workflow_schema": workflow_schema[0] if type(workflow_schema) == tuple else workflow_schema,
                })
        return workflow_instance[0] if type(workflow_instance) == tuple else workflow_instance

    def is_original(self):
        """
        Возвращает Истина, если документ изготовлен локальным нодом,
        и возвращает Ложь если документ является репликой оригинала
        """
        return self.created_by == settings.ELASTIC_SETTINGS['server_name']

    @transaction.atomic()
    def hold_at_rest_register(self):
        """
        Проводим коносамент по регистру остатков
        """

        # Оформляем расход
        try:
            from_location = StockLocation.objects.get(code=self.from_location_code)
        except ObjectDoesNotExist:
            pass
        else:
            for row in self.rows.iterator():
                try:
                    product = Product.objects.get(sku=row.product_sku)
                except ObjectDoesNotExist:
                    raise ProductNotExists(row.product_sku)

                register = RestRegister.objects.get_or_create(
                        product=product, location=from_location,
                        defaults={
                            "actual_datetime": timezone.datetime(1970, 1, 1, tzinfo=timezone.get_current_timezone()),
                            "location": from_location,
                            "product": product,
                        })
                RestRegisterRow.objects.create(
                    effective_date=self.effective_date,
                    price_cost=None,  # При оформлении расхода цены не пересчитываются
                    quantity=-row.quantity,
                    doc_object=self,
                    register=register[0] if type(register) == tuple else register
                )

        # Оформляем приход
        try:
            to_location = StockLocation.objects.get(code=self.to_location_code)
        except ObjectDoesNotExist:
            pass
        else:
            for row in self.rows.iterator():
                # Ищем товар по складскому коду из накладной
                try:
                    product = Product.objects.get(sku=row.product_sku)
                except ObjectDoesNotExist:
                    raise ProductNotExists(row.product_sku)

                register = RestRegister.objects.get_or_create(
                    product__sku=row.product_sku,
                    location=to_location,
                    product=product,
                    defaults={
                        "actual_datetime": timezone.datetime(1970, 1, 1, tzinfo=timezone.get_current_timezone()),
                        "location": to_location,
                        "product": product,
                    })

                RestRegisterRow.objects.create(
                    effective_date=self.effective_date,
                    price_cost=row.price_cost,
                    quantity=row.quantity,
                    doc_object=self,
                    register=register[0] if type(register) == tuple else register
                )

    @transaction.atomic()
    def unhold_at_rest_register(self):
        """
        Отмена проводки коносамента по регистру остатков
        """

        for register in RestRegisterRow.objects.filter(
                doc_id=self.id,
                doc_content_type=ContentType.objects.get_for_model(Consignment)):
            register.delete()

    @transaction.atomic()
    def hold_at_transit_register(self):
        """
        Провести коносамент по регистру "Товары в пути"
        """
        try:
            to_location = StockLocation.objects.get(code=self.to_location_code)
        except ObjectDoesNotExist:
            pass
        else:
            for row in self.rows.iterator():
                # Ищем товар по складскому коду из накладной
                try:
                    product = Product.objects.get(sku=row.product_sku)
                except ObjectDoesNotExist:
                    raise ProductNotExists(row.product_sku)

                register = TransitRegister.objects.get_or_create(
                    product__sku=row.product_sku,
                    location=to_location,
                    product=product,
                    defaults={
                        "actual_datetime": timezone.datetime(1970, 1, 1, tzinfo=timezone.get_current_timezone()),
                        "location": to_location,
                        "product": product,
                    })

                TransitRegisterRow.objects.create(
                    effective_date=timezone.now(),
                    quantity=row.quantity,
                    doc_object=self,
                    register=register[0] if type(register) == tuple else register
                )

    @transaction.atomic()
    def unhold_at_transit_register(self):
        """
        Отменить проводку коносамента по регистру "Товары в пути"
        """

        for register in TransitRegisterRow.objects.filter(
                doc_id=self.id,
                doc_content_type=ContentType.objects.get_for_model(Consignment)):
            register.delete()

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):

        if not self.code:
            self.code = uuid.uuid1()

        # Устанавливаем дату создания
        if not self.creation_date:
            self.creation_date = timezone.now()

        # Устанавливаем дату последней модификации
        if not self.last_modify_date:
            self.last_modify_date = timezone.now()

        if not self.doc_no:
            # Устанавливаем номер документа
            s = Sequence.objects.get_or_create(
                name='consignment',
                defaults={
                    'name': 'consignment',
                    'prefix': 'CON',
                    'separator': '-',
                }
            )[0]
            self.doc_no = s.request_next_number()

        super(Consignment, self).save(force_insert, force_update, using, update_fields)


class ConsignmentRow(models.Model):
    """
    Табличная часть ТОРГ-13 "Накладная на внутреннее перемещение, передачу товаров, тары"
    """
    class Meta:
        verbose_name = _('Consignment row')
        verbose_name_plural = _('Consignment rows')

    product = models.CharField(max_length=1024, verbose_name=_('Product name'))
    product_sku = models.CharField(verbose_name=_('Product SKU'), max_length=128)
    quantity = models.DecimalField(max_digits=12, decimal_places=3, verbose_name=_('Quantity'))
    price_cost = models.DecimalField(max_digits=15, decimal_places=2, blank=True, null=True, default=0,
                                     verbose_name=_('Price cost'))
    total_cost = models.DecimalField(max_digits=15, decimal_places=2, blank=True, null=True, default=0,
                                     verbose_name=_('Total cost'))
    consignment = models.ForeignKey(Consignment, related_name='rows', verbose_name=_('Consignment'))

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        super(ConsignmentRow, self).save(force_insert, force_update, using, update_fields)
        self.consignment.get_workflow().delete()
        self.consignment.get_workflow().start()


class RevaluationAct(models.Model):
    """
    Акт переоценки товара
    """
    class Meta:
        verbose_name = _('Act of revaluation')
        verbose_name_plural = _("Acts of revaluation")

    #: Номер документа
    doc_no = models.CharField(verbose_name=_('Doc no'), max_length=128, db_index=True)

    #: Внутренний код документа
    code = models.CharField(max_length=128, verbose_name=_('Code'))

    #: Дата/время создания
    creation_date = models.DateTimeField(verbose_name=_('Creation date'))

    #: Дата исполнения
    effective_date = models.DateTimeField(null=True, blank=True, verbose_name=_('Effective date'))

    #: Дата/время последней модификации
    last_modify_date = models.DateTimeField(verbose_name=_('Last modify date'))

    #: Метка узла-создателя документа
    created_by = models.CharField(max_length=64, verbose_name=_('Node creator'))

    location_name = models.CharField(max_length=255, verbose_name=_('Location name'))
    location_code = models.CharField(max_length=128, verbose_name=_('Location code'))
    description = models.TextField(null=True, blank=True, verbose_name=_('Description'))

    def __unicode__(self):
        tmpl = Template("{% load tz %} {{ effective_date|localtime}}")
        creation_date = tmpl.render(Context({'effective_date': self.creation_date}))
        return u"#{0} {1} {2}".format(self.doc_no, _('at'), creation_date)

    def get_workflow(self):
        # Регистрируем/загружаем связанный поток исполнения
        workflow_schema = Schema.objects.get_or_create(
                code='goods_revaluation',
                defaults={
                    "name": 'Переоценка товара',
                    "code": 'goods_revaluation',
                })

        workflow_instance = Workflow.objects.get_or_create(
                doc_id=self.id,
                defaults={
                    "doc_object": self,
                    "workflow_schema": workflow_schema[0] if type(workflow_schema) == tuple else workflow_schema,
                })
        return workflow_instance[0] if type(workflow_instance) == tuple else workflow_instance

    def is_original(self):
        """
        Возвращает Истина, если документ изготовлен локальным нодом,
        и возвращает Ложь если документ является репликой оригинала
        """
        return self.created_by == settings.ELASTIC_SETTINGS['server_name']

    @transaction.atomic()
    def hold_at_rest_register(self):
        """
        Проводим акт переоценки товара устанавливая цены на регистрах
        """

        try:
            location = StockLocation.objects.get(code=self.location_code)
        except ObjectDoesNotExist:
            pass
        else:
            for row in self.rows.all():
                try:
                    product = Product.objects.get(sku=row.product_sku)
                except ObjectDoesNotExist:
                    raise ProductNotExists(row.product_sku)

                register = RestRegister.objects.get_or_create(
                        product=product, location=location,
                        defaults={
                            "actual_datetime": timezone.datetime(1970, 1, 1, tzinfo=timezone.get_current_timezone()),
                            "location": location,
                            "product": product,
                        })

                RestRegisterRow.objects.create(
                    effective_date=self.effective_date,
                    price_cost=row.price_cost,
                    quantity=0,
                    doc_object=self,
                    register=register[0] if type(register) == tuple else register
                )

    @transaction.atomic()
    def unhold_at_rest_register(self):
        """
        Отмена проводки акта переоценки товара
        """

        for register in RestRegisterRow.objects.filter(
                doc_id=self.id,
                doc_content_type=ContentType.objects.get_for_model(RevaluationAct)):
            register.delete()

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):

        # Устанавливаем дату создания и дату последней модификации
        if not self.creation_date:
            self.creation_date = timezone.now()

        if not self.last_modify_date:
            self.last_modify_date = timezone.now()

        if not self.code:
            self.code = uuid.uuid1()

        if not self.doc_no:
            # Устанавливаем номер документа
            s = Sequence.objects.get_or_create(
                name='revaluation_act',
                defaults={
                    'name': 'revaluation_act',
                    'prefix': 'RVL',
                    'separator': '-',
                }
            )[0]
            self.doc_no = s.request_next_number()

        super(RevaluationAct, self).save(force_insert, force_update, using, update_fields)


class RevaluationActRow(models.Model):
    """
    Строка табличной части Акта переоценки товара
    """
    class Meta:
        verbose_name = _('Row of act of revaluation')
        verbose_name_plural = _('Rows of act of revaluation')

    product = models.CharField(max_length=1024, verbose_name=_('Product name'))
    product_sku = models.CharField(verbose_name=_('Product SKU'), max_length=128)
    price_cost = models.DecimalField(max_digits=15, decimal_places=2, verbose_name=_('Price cost'))
    revaluation_act = models.ForeignKey(RevaluationAct, related_name='rows', verbose_name=_('Act of revaluation'))

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        super(RevaluationActRow, self).save(force_insert, force_update, using, update_fields)
        self.revaluation_act.get_workflow().delete()
        self.revaluation_act.get_workflow().start()


class Register(models.Model):
    """
    Регистр информации
    """
    class Meta:
        abstract = True

    actual_datetime = models.DateTimeField(verbose_name=_('Actual date and time'))
    actual_quantity = models.DecimalField(max_digits=12, decimal_places=3, null=True, blank=False, default=0,
                                          verbose_name=_('Actual quantity'))
    location = models.ForeignKey(StockLocation, limit_choices_to={"type_location": 'WH'},
                                 verbose_name=_('Stock location'))
    product = models.ForeignKey(Product, verbose_name=_('Product'))

    def product_sku(self):
        return unicode(self.product.sku)
    product_sku.short_description = _('SKU')

    def manufacturer(self):
        return unicode(self.product.manufacturer)
    manufacturer.short_description = _('Manufacturer')

    def __unicode__(self):
        return unicode(self.product)


class RegisterRow(models.Model):
    """
    Строка регистра информации
    """
    class Meta:
        abstract = True

    #: Дата/время создания проводки
    creation_date = models.DateTimeField(auto_now_add=True, verbose_name=_('Creation date'))

    #: Дата/время исполнения
    effective_date = models.DateTimeField(verbose_name=_('Effective date'))

    #: Количество
    quantity = models.DecimalField(max_digits=12, decimal_places=3, verbose_name=_('Quantity'))

    # Корреспондирующий документ
    doc_content_type = models.ForeignKey(ContentType)
    doc_id = models.PositiveIntegerField()
    doc_object = GenericForeignKey('doc_content_type', 'doc_id')

    def original_document(self):
        language = getattr(settings, 'LANGUAGE_CODE', 'en')
        with translation.override(language):
            return u"{0} {1}".format(
                translation.ugettext(self.doc_object._meta.verbose_name.capitalize()), self.doc_object)
    original_document.short_description = _('Original document')


class RestRegister(Register):
    """
    Регистр остатков. Предназначен для сохранения стоимости и остатков товара на конкретном месте хранения
    """
    class Meta:
        verbose_name = _('Rest register')
        verbose_name_plural = _('Rest registers')

    actual_price_cost = models.DecimalField(max_digits=15, decimal_places=2, null=True, blank=True,
                                            verbose_name=_('Actual price cost'))

    def re_calculate(self):
        # Чистим от строк с битыми ссылками на документ-источник
        for row in RestRegisterRow.objects.filter(register=self):
            if not row.doc_object:
                row.delete()

        # Если после чистки не осталось строк, то удаляем сам регистр
        if RestRegisterRow.objects.filter(register=self).count() == 0:
            self.delete()
            return

        # Инициализируем регистр начальными значениями
        self.actual_price_cost = 0
        self.actual_quantity = 0

        for row in RestRegisterRow.objects.filter(register=self).order_by("effective_date"):

            if row.price_cost:
                # Устанавливаем цену
                self.actual_price_cost = row.price_cost

            self.actual_quantity += row.quantity
            self.actual_datetime = row.effective_date

        self.save()


class RestRegisterRow(RegisterRow):
    """
    Строка регистра остатков.
    """
    class Meta:
        verbose_name = _('Rest register row')
        verbose_name_plural = _('Rest register rows')

    price_cost = models.DecimalField(max_digits=15, decimal_places=2, null=True, blank=True,
                                     verbose_name=_('Price cost'))
    register = models.ForeignKey(RestRegister, related_name='rows', verbose_name=_('Register'))

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        if not self.pk:
            if self.effective_date > self.register.actual_datetime and self.quantity >= 0 and \
                    self.price_cost is not None:
                # Если приход (quantity > 0) или акт установки цен (quantity == 0)
                # и не возврат товара (quantity > 0 и price_cost == None)
                # то устанвливаем цену на регистре
                self.register.actual_datetime = self.effective_date
                self.register.actual_price_cost = self.price_cost
            self.register.actual_quantity += self.quantity
            self.register.save()
        super(RestRegisterRow, self).save(force_insert, force_update, using, update_fields)

    def delete(self, using=None):
        if not self.doc_object:
            super(RestRegisterRow, self).delete(using)
            return

        # Откатываем остатки
        self.register.actual_quantity -= self.quantity
        self.register.save()
        super(RestRegisterRow, self).delete(using)

        # Получаем коллекцию оставшихся строк регистра, сортированных по дате
        rows = RestRegisterRow.objects.filter(register=self.register).order_by("-effective_date")

        # Если строк больше нет....
        if not rows:
            self.register.delete()  # ...тогда удаляем регистр
        else:
            # Если строки есть, то из первой из них (с установленной ценой) берем актуальную цену и дату
            for row in rows:
                if row.price_cost:
                    self.register.actual_datetime = row.effective_date
                    self.register.actual_price_cost = row.price_cost
                    self.register.save()
                    break


class ReserveRegister(Register):
    """
    Регистр резерва. Предназначен для сохранения информации о резервах
    """
    class Meta:
        verbose_name = _('Reserve register')
        verbose_name_plural = _('Reserve registers')


class ReserveRegisterRow(RegisterRow):
    """
    Строка регистра резерва.
    """
    class Meta:
        verbose_name = _('Reserve register row')
        verbose_name_plural = _('Reserve register rows')

    register = models.ForeignKey(ReserveRegister, related_name='rows', verbose_name=_('Register'))

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        if not self.pk:
            if self.effective_date > self.register.actual_datetime:
                self.register.actual_datetime = self.effective_date
            self.register.actual_quantity += self.quantity
            self.register.save()
        super(ReserveRegisterRow, self).save(force_insert, force_update, using, update_fields)

    def delete(self, using=None):
        if not self.doc_object:
            super(ReserveRegisterRow, self).delete(using)
            return

        # Откатываем остатки
        self.register.actual_quantity -= self.quantity
        self.register.save()
        super(ReserveRegisterRow, self).delete(using)

        # Получаем коллекцию оставшихся строк регистра, сортированных по дате
        rows = ReserveRegisterRow.objects.filter(register=self.register).order_by("-effective_date")

        # Если строк больше нет....
        if not rows:
            self.register.delete()  # ...тогда удаляем регистр
        else:
            # Если строки есть, то из первой из них берем актуальную дату
            for row in rows:
                self.register.actual_datetime = row.effective_date
                self.register.save()
                break


class TransitRegister(Register):
    """
    Регистр "Товары в пути". Предназначен отслеживания товаров, поступление которых мы ждем
    """
    class Meta:
        verbose_name = _('Transit register')
        verbose_name_plural = _('Transit registers')


class TransitRegisterRow(RegisterRow):
    """
    Строка регистра "Товары в пути".
    """
    class Meta:
        verbose_name = _('Transit register row')
        verbose_name_plural = _('Transit register rows')

    register = models.ForeignKey(TransitRegister, related_name='rows', verbose_name=_('Register'))

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        if not self.pk:
            self.register.actual_quantity += self.quantity
            self.register.save()
        super(TransitRegisterRow, self).save(force_insert, force_update, using, update_fields)

    def delete(self, using=None):
        if not self.doc_object:
            super(TransitRegisterRow, self).delete(using)
            return

        # Откатываем остатки
        self.register.actual_quantity -= self.quantity
        self.register.save()
        super(TransitRegisterRow, self).delete(using)

        # Получаем коллекцию оставшихся строк регистра, сортированных по дате
        rows = TransitRegisterRow.objects.filter(register=self.register).order_by("-effective_date")

        # Если строк больше нет....
        if not rows:
            self.register.delete()  # ...тогда удаляем регистр
        else:
            # Если строки есть, то из первой из них берем актуальную дату
            for row in rows:
                self.register.actual_datetime = row.effective_date
                self.register.save()
                break


class Sequence(models.Model):
    class Meta:
        verbose_name = _('Sequence')
        verbose_name_plural = _('Sequences')

    name = models.CharField(max_length=255, verbose_name=_('Name'))
    separator = models.CharField(default='-', blank=True, max_length=1, verbose_name=_('Separator'))
    prefix = models.CharField(max_length=8, null=True, blank=True, verbose_name=_('Prefix'))
    number = models.IntegerField(editable=False, default=0, verbose_name=_('Number'))
    suffix = models.CharField(max_length=8, null=True, blank=True, verbose_name=_('Suffix'))

    @transaction.atomic
    def request_next_number(self):
        s = Sequence.objects.select_for_update().get(id=self.id)
        return s.get_next_number()

    def current_number(self):
        if self.prefix and self.suffix:
            return u"{0}{3}{1}{3}{2}".format(self.prefix, self.number, self.suffix, self.separator)
        if self.prefix:
            return u"{0}{2}{1}".format(self.prefix, self.number, self.separator)
        if self.suffix:
            return u"{0}{2}{1}".format(self.number, self.suffix, self.separator)
        return u"{0}".format(self.number)
    current_number.short_description = _('Current number')

    def get_next_number(self, force_insert=False, force_update=False, using=None):
        self.number += 1
        super(Sequence, self).save(force_insert, force_update, using)
        return self.current_number()

    def reset_sequence(self, force_insert=False, force_update=False, using=None):
        self.number = 0
        super(Sequence, self).save(force_insert, force_update, using)

    def __unicode__(self):
        return self.name
